*******************************************************************************
*   Title: draft-stimulus.do
* Purpose: Produce figures and tables for final published version.
*   Owner: Eric Zwick
*    Date: 2016-08-01
*******************************************************************************

* Import programs
do $dodir/make-20150730.do

*******************************************************************************
* Bunching analysis
*******************************************************************************
capture program drop load_bunch_data/*%<*/
program define load_bunch_data

    use $analysis/ibunch.dta, clear

    g loss = net_incm_plus < 0
    label define loss1 0 "Net Income Plus Depreciation >= 0" 1 "Net Income Plus Depreciation < 0"
    label values loss loss1

    g has_nol = avlbl_crryovr_nol > 0 & avlbl_crryovr_nol != .

    count
    tab year
    
end/*%>*/

capture program drop graph_bunch_calendar/*%<*/
program define graph_bunch_calendar

    bunch_pic s179eb if year < 1997, lower(10) upper(40) year(pre1997) cutoff(17.5)
    bunch_pic s179eb if year == 1997, lower(10) upper(40) year(1997) cutoff(18)
    bunch_pic s179eb if year == 1998, lower(10) upper(40) year(1998) cutoff(18.5)
    bunch_pic s179eb if year == 1999, lower(10) upper(40) year(1999) cutoff(19)
    bunch_pic s179eb if year == 2000, lower(10) upper(40) year(2000) cutoff(20)
    bunch_pic s179eb if year == 2001 | year == 2002, lower(10) upper(40) year(2001-02) cutoff(24)
    bunch_pic s179eb if year == 2003, lower(50) upper(150) year(2003) cutoff(100)
    bunch_pic s179eb if year == 2004, lower(50) upper(150) year(2004) cutoff(102)
    bunch_pic s179eb if year == 2005, lower(50) upper(150) year(2005) cutoff(105)
    bunch_pic s179eb if year == 2006, lower(50) upper(150) year(2006) cutoff(108)
    bunch_pic s179eb if year == 2007, lower(60) upper(160) year(2007) cutoff(125)
    bunch_pic s179eb if year == 2008 | year == 2009, lower(180) upper(280) year(2008-09) cutoff(250)
    bunch_pic s179eb if year == 2010, lower(300) upper(550) year(2010) cutoff(500)

end/*%>*/

capture program drop graph_bunch_loss/*%<*/
program define graph_bunch_loss

    preserve
    * keep if year > 2002
    bunch_split loss
    bunch_est, mark(loss) by(loss)
    restore
    
end/*%>*/

capture program drop graph_bunch_nol/*%<*/
program define graph_bunch_nol

    preserve
    keep if has_nol & net_incm_plus > 0 & net_incm_plus != .
    g nol_income = avlbl_crryovr_nol/net_incm_plus
    trim_tails nol_income, level(.01)
    label variable nol_income "NOL Stock/Net Income Plus Depreciation"
    xtile nol_group = nol_income, nq(10)
    bunch_split nol_group
    bunch_est, mark(nol) by(nol_group) plot(1) xlabel("Loss Carryforward Stock")
    restore

end/*%>*/

capture program drop graph_bunch_bandwidth/*%<*/
program define graph_bunch_bandwidth

    preserve
    tempfile bunch_nol
    set matsize 800
    egen ni_bins = cut(net_incm_plus), at(-250(5)250)
    drop if ni_bins == .
    save `bunch_nol'
    matrix loss_b = (.)
    matrix loss_se = (.)
    matrix noloss_b = (.)
    matrix noloss_se = (.)
    forvalues i = 1/50 {
        use `bunch_nol', clear
        keep if abs(ni_bins) <= `i'*5
        bunch_split loss
        bunch_est, mark(loss) by(loss) delta(2)
        qui sum bc_bunch if loss == 1, meanonly
        matrix loss_b = loss_b,(`r(mean)')
        qui sum bc_bunch if loss == 0, meanonly
        matrix noloss_b = noloss_b,(`r(mean)')
        qui sum bc_se if loss == 1, meanonly
        matrix loss_se = loss_se,(`r(mean)')
        qui sum bc_se if loss == 0, meanonly
        matrix noloss_se = noloss_se,(`r(mean)')
    }
    clear
    set obs 50
    gen width = .
    gen b_loss = .
    gen se_loss = .
    gen b_noloss = .
    gen se_noloss = .
    forvalues i = 1/50 {
        replace width = `i'*5 if _n == `i'
        replace b_loss = loss_b[1,`i'+1] if _n == `i'
        replace se_loss = loss_se[1,`i'+1] if _n == `i'
        replace b_noloss = noloss_b[1,`i'+1] if _n == `i'
        replace se_noloss = noloss_se[1,`i'+1] if _n == `i'
    }
    gen b_loss_max = b_loss + 1.96*se_loss
    gen b_loss_min = b_loss - 1.96*se_loss
    gen b_noloss_max = b_noloss + 1.96*se_noloss
    gen b_noloss_min = b_noloss - 1.96*se_noloss

    #delimit ;
    twoway (rarea b_loss_max b_loss_min width, fcolor(none)) 
        (connected b_loss width, symbol(o))
        (rarea b_noloss_max b_noloss_min width, lcolor(green) fcolor(none)) 
        (connected b_noloss width, mfcolor(green) symbol(O)),
        xtitle("Bandwidth (000s)") ytitle("Bunch Ratio")
        legend(order(4 2) 
               label(2 "Net Income Plus Depreciation < 0")
               label(4 "Net Income Plus Depreciation >= 0"));
    #delimit cr
    graph export bunch_rd.eps, replace
    restore

end/*%>*/

*******************************************************************************
* Bonus analysis
*******************************************************************************
capture program drop load_bonus_data/*%<*/
program define load_bonus_data 
    syntax, [dset(string) filter(string)]

    if ("`dset'" != "") {
        use "$analysis/`dset'", clear
    } 
    else if ("`filter'" == "raw" | "`filter'" == "raw_noyr") {
        if ("`filter'" == "raw_noyr") {
            load_investment, filter("raw_noyr")
        }
        else {
            load_investment
        }
        gen_returns
        
        duplicates drop ein year end_acct_mon, force
        
        * Merge on better weights.
        merge 1:1 ein year end_acct_mon using "$analysis/soi_weights.dta", keep(1 3) nogen
        rename weight_f weight_f_ota    
        gen weight_f = cond(weight_f_soi != ., weight_f_soi, weight_f_ota)
    }
    else {
        use $analysis/ibonus.dta, clear

        * Merge on better weights.
        merge 1:1 ein year end_acct_mon using "$analysis/soi_weights.dta", keep(1 3) nogen
        rename weight_f weight_f_ota    
        gen weight_f = cond(weight_f_soi != ., weight_f_soi, weight_f_ota)
    }

    if ("`filter'" == "none" | "`filter'" == "raw" | "`filter'" == "raw_noyr") {
        display "keeping all"
    }
    else {
        display "keeping big"
        keep if big == 1
    }
end/*%>*/

capture program drop graph_bonus_calendar/*%<*/
program define graph_bonus_calendar
    syntax, [dset(string)]

    tempfile orig
    save `orig'
    if ("`dset'" != "") {
        use "$analysis/`dset'", clear
    }
    else {
        use $analysis/ibonus.dta, clear
    }
    
    gen treatment = cond(wpv_treatment2 <= 3, 1, cond(wpv_treatment2 >= 8, 0, .))
    mkspline size_spline 10 = std_tot_assts, pctile
    mkspline sales_spline 10 = grs_rcpts, pctile
    mkspline prof_spline 10 = profit_margin, pctile
    mkspline age_spline 10 = age, pctile
    
    local regspec treatment size_spline* sales_spline* prof_spline* age_spline*
    foreach x of varlist log_investment log_odds investment_cst {
        #delimit ;
        ddplot_yagan `x' `regspec' if big & treatment != . [aweight=dfl_weight], 
            start(1996) diag(1) raj(1) end(2004) mark(dd-z-early);
        ddplot_yagan `x' `regspec' if big & treatment != . [aweight=dfl_weight], 
            start(2005) diag(1) raj(1) end(2010) mark(dd-z-late);
        #delimit cr
    }
    drop treatment 

    use `orig', clear
end/*%>*/

capture program drop graph_bonus_coeffsplit/*%<*/
program define graph_bonus_coeffsplit

    preserve
    * Sales.
    bys mein: egen mean_size = mean(grs_rcpts)
    drop size_group
    drop if mean_size == 0 | mean_size == .
    xtile size_group = mean_size, nq(10)
    gen beta_one = .
    gen beta_one_min = .
    gen beta_one_max = .
    gen beta_two = .
    gen beta_two_min = .
    gen beta_two_max = .
    gen beta_three = .
    gen beta_three_min = .
    gen beta_three_max = .

    forval i = 1/10 {
        areg log_investment wpv_policy _Iyear* if size_group == `i', absorb(mein) vce(cluster mein)
        matrix B = e(b)
        matrix V = e(V)
        replace beta_one = B[1,1] if size_group == `i'
        replace beta_one_min = B[1,1] - 1.96*V[1,1]^.5 if size_group == `i'
        replace beta_one_max = B[1,1] + 1.96*V[1,1]^.5 if size_group == `i'
        areg log_odds wpv_policy _Iyear* if size_group == `i', absorb(mein) vce(cluster four_digit)
        matrix B = e(b)
        matrix V = e(V)
        replace beta_two = B[1,1] if size_group == `i'
        replace beta_two_min = B[1,1] - 1.96*V[1,1]^.5 if size_group == `i'
        replace beta_two_max = B[1,1] + 1.96*V[1,1]^.5 if size_group == `i'
        areg investment_cst aftertax _Iyear* if size_group == `i', absorb(mein) vce(cluster mein)    
        matrix B = e(b)
        matrix V = e(V)
        replace beta_three = B[1,1] if size_group == `i'
        replace beta_three_min = B[1,1] - 1.96*V[1,1]^.5 if size_group == `i'
        replace beta_three_max = B[1,1] + 1.96*V[1,1]^.5 if size_group == `i'
    }
    collapse mean_size beta_*, by(size_group)
    #delimit ;
    twoway (rcap beta_one_min beta_one_max size_group) 
        (scatter beta_one size_group, msymbol(o) msize(large)), 
        ytitle("Elasticity") xtitle("Size Bins (Sales)") legend(off) xsize(6);
    graph export "size_coeffs-int.eps", replace;
    twoway (rcap beta_two_min beta_two_max size_group) 
        (scatter beta_two size_group, msymbol(o) msize(large)), 
        ytitle("Elasticity") xtitle("Size Bins (Sales)") legend(off) xsize(6);
    graph export "size_coeffs-ext.eps", replace;
    twoway (rcap beta_three_min beta_three_max size_group) 
        (scatter beta_three size_group, msymbol(o) msize(large)), 
        ytitle("User Cost Elasticity") xtitle("Size Bins (Sales)") legend(off) xsize(6);
    graph export "size_coeffs-scaled.eps", replace;
    #delimit cr
    restore
    
end/*%>*/

capture program drop graph_bonus_bandwidth/*%<*/
program define graph_bonus_bandwidth

    preserve
    tempfile bonus_nol
    set matsize 800
    egen ni_bins = cut(net_incm_plus), at(-500(5)500)
    drop if ni_bins == .
    save `bonus_nol', replace
    matrix loss_b = (.)
    matrix loss_se = (.)
    matrix noloss_b = (.)
    matrix noloss_se = (.)
    forvalues i = 1/100 {
        use `bonus_nol', clear
        keep if abs(ni_bins) <= `i'*5
        areg log_investment taxable##(c.wpv_policy year), absorb(mein) vce(cluster mein)
        matrix loss_b = loss_b,(_b[wpv_policy])
        matrix loss_se = loss_se,(_se[wpv_policy])
        lincom 1.taxable#c.wpv_policy + wpv_policy
        matrix noloss_b = noloss_b,(`r(estimate)')
        matrix noloss_se = noloss_se,(`r(se)')
    }
    clear
    set obs 100
    gen width = .
    gen b_loss = .
    gen se_loss = .
    gen b_noloss = .
    gen se_noloss = .
    forvalues i = 1/100 {
        replace width = `i'*5 if _n == `i'
        replace b_loss = loss_b[1,`i'+1] if _n == `i'
        replace se_loss = loss_se[1,`i'+1] if _n == `i'
        replace b_noloss = noloss_b[1,`i'+1] if _n == `i'
        replace se_noloss = noloss_se[1,`i'+1] if _n == `i'
    }
    gen b_loss_max = b_loss + 1.96*se_loss
    gen b_loss_min = b_loss - 1.96*se_loss
    gen b_noloss_max = b_noloss + 1.96*se_noloss
    gen b_noloss_min = b_noloss - 1.96*se_noloss

    #delimit ;
    twoway (rarea b_loss_max b_loss_min width, fcolor(none)) 
        (connected b_loss width, symbol(o))
        (rarea b_noloss_max b_noloss_min width, lcolor(green) fcolor(none)) 
        (connected b_noloss width, mfcolor(green) symbol(O)),
        xtitle("Bandwidth (000s)") ytitle("Elasticity")
        legend(order(4 2) 
               label(2 "Net Income Plus Depreciation < 0")
               label(4 "Net Income Plus Depreciation >= 0"));
    #delimit cr
    graph export bonus_rd.eps, replace
    restore

end/*%>*/

capture program drop numbers_bonus/*%<*/
program define numbers_bonus

    * Summarize z in bonus and non-bonus years.
    sum wpv_policy if year < 2001
    sum wpv_policy if year >= 2001 & year <= 2004
    sum wpv_policy if year >= 2008

    * Industries in each wpv group.
    tab two_digit if wpv_treatment2 <= 3, sort
    tab two_digit if wpv_treatment2 >= 8, sort


end/*%>*/

capture program drop table_bonus_sumstats/*%<*/
program define table_bonus_sumstats

    foreach x of varlist s179eb log_investment log_odds investment_cst dprcbl_assts wpv_policy wfd_policy ///
        std_tot_assts grs_rcpts wage_compensation profit_margin  net_incm_plus ///
        cash_flow_cst log_payroll log_structures depr_asset_growth ///
        rent_growth payroll_growth debt_issues {
            egen `x'_group = cut(`x'), group(100)
            egen `x'_p10h = mean(`x') if abs(`x'_group - 10) <= 1
            egen `x'_p10 = max(`x'_p10h)
            egen `x'_p50h = mean(`x') if abs(`x'_group - 50) <= 1
            egen `x'_p50 = max(`x'_p50h)
            egen `x'_p90h = mean(`x') if abs(`x'_group - 90) <= 1
            egen `x'_p90 = max(`x'_p90h)
            egen `x'_count = count(`x')
            estpost tabstat `x' `x'_p10 `x'_p50 `x'_p90 `x'_count, listwise statistics(mean)
            esttab using `x'-sum.tex, cells("`x' `x'_p10 `x'_p50 `x'_p90 `x'_count") nomtitle nonumber label replace booktabs fragment b(a2)
            drop `x'_*
    }
    sum has_eligible dividends payer log_dividends has_nol  has_bonus avlbl_crryovr_nol taxable

end/*%>*/

capture program drop table_bonus_indy/*%<*/
program define table_bonus_indy

    ** Main regressions.
    eststo: areg log_investment wpv_policy _Iyear*, absorb(mein) vce(cluster mein)
    eststo: areg log_investment wpv_policy cash_flow_cst _Iyear*, absorb(mein) vce(cluster mein)
    eststo: areg log_investment wpv_policy _Iyear* if year < 2005, absorb(mein) vce(cluster mein)
    eststo: areg log_investment wpv_policy _Iyear* if year >= 2005, absorb(mein) vce(cluster mein)
    estadd local controls "No": est*
    #delimit ;
    eststo: areg log_investment wpv_policy _Iyear* industry_q
        c.std_tot_assts##c.std_tot_assts##c.std_tot_assts##c.std_tot_assts
        c.grs_rcpts##c.grs_rcpts##c.grs_rcpts##c.grs_rcpts
        c.profit_margin##c.profit_margin##c.profit_margin##c.profit_margin
        c.age##c.age##c.age##c.age,
        absorb(mein) vce(cluster mein);
    #delimit cr
    estadd local controls "Yes": est5
    estadd local indy "No": est*
    eststo: areg log_investment wpv_policy _Iyear* two_digit_naics##(c.year##c.year), absorb(mein) vce(cluster mein)
    estadd local indy "Yes": est6
    estadd local firm "Yes": est*
    estadd local year "Yes": est*
    #delimit ;
    esttab using bonus-indy-20130818.tex,
        label replace se booktabs nonumbers fragment b(a2)
            mgroups("LHS Variable is Log(Eligible Investment)",
            prefix(\multicolumn{@span}{c}{) suffix(})
                span erepeat(\cmidrule(lr){@span}))
            mtitles("All" "All" "Pre-2005" "Post-2004" "All" "All")
            drop(_I* c.*)
            scalars("N Observations" "N_clust Firms" "r2 R$^2$" 
                "year Year Effects" "firm Firm Effects" 
                "controls Controls" "indy Industry Trends");
    #delimit cr
    estimates clear

    ** Now for extensive margin.
    ** Main regressions.
    eststo: areg log_odds wpv_policy _Iyear*, absorb(mein) vce(cluster four_digit)
    eststo: areg log_odds wpv_policy cash_flow_cst _Iyear*, absorb(mein) vce(cluster four_digit)
    eststo: areg log_odds wpv_policy _Iyear* if year < 2005, absorb(mein) vce(cluster four_digit)
    eststo: areg log_odds wpv_policy _Iyear* if year >= 2005, absorb(mein) vce(cluster four_digit)
    estadd local controls "No": est*
    #delimit ;
    eststo: areg log_odds wpv_policy _Iyear* industry_q
        c.std_tot_assts##c.std_tot_assts##c.std_tot_assts##c.std_tot_assts
        c.grs_rcpts##c.grs_rcpts##c.grs_rcpts##c.grs_rcpts
        c.profit_margin##c.profit_margin##c.profit_margin##c.profit_margin
        c.age##c.age##c.age##c.age,
        absorb(mein) vce(cluster four_digit);
    #delimit cr
    estadd local controls "Yes": est5
    estadd local indy "No": est*
    eststo: areg log_odds wpv_policy _Iyear* two_digit_naics##(c.year##c.year), absorb(mein) vce(cluster four_digit)
    estadd local indy "Yes": est6
    estadd local firm "Yes": est*
    estadd local year "Yes": est*
    #delimit ;
    esttab using bonus-indy-extensive-20130818.tex,
        label replace se booktabs nonumbers fragment b(a2)
            mgroups("LHS Variable is Log(P(Eligible Investment > 0))",
            prefix(\multicolumn{@span}{c}{) suffix(})
                span erepeat(\cmidrule(lr){@span}))
            mtitles("All" "All" "Pre-2005" "Post-2004" "All" "All")
            drop(_I* c.*)
            scalars("N Observations" "N_clust Firms" "r2 R$^2$" 
                "year Year Effects" "firm Firm Effects"
                "controls Controls" "indy Industry Trends");        
    #delimit cr
    estimates clear

    ** Now for investment/lagged assets.
    eststo: areg investment_cst aftertax _Iyear*, absorb(mein) vce(cluster mein)
    eststo: areg investment_cst aftertax cash_flow_cst _Iyear*, absorb(mein) vce(cluster mein)
    eststo: areg investment_cst aftertax _Iyear* if year < 2005, absorb(mein) vce(cluster mein)
    eststo: areg investment_cst aftertax _Iyear* if year >= 2005, absorb(mein) vce(cluster mein)
    estadd local controls "No": est*
    #delimit ;
    eststo: areg investment_cst aftertax _Iyear* industry_q
        c.std_tot_assts##c.std_tot_assts##c.std_tot_assts##c.std_tot_assts
        c.grs_rcpts##c.grs_rcpts##c.grs_rcpts##c.grs_rcpts
        c.profit_margin##c.profit_margin##c.profit_margin##c.profit_margin
        c.age##c.age##c.age##c.age,
        absorb(mein) vce(cluster mein);
    #delimit cr
    estadd local controls "Yes": est5
    estadd local indy "No": est*
    eststo: areg investment_cst aftertax _Iyear* two_digit_naics##(c.year##c.year), absorb(mein) vce(cluster mein)
    estadd local indy "Yes": est6
    estadd local firm "Yes": est*
    estadd local year "Yes": est*

    #delimit ;
    esttab using bonus-indy-scaled-20130818.tex,
        label replace se booktabs nonumbers fragment b(a2)
            mgroups("LHS Variable is Eligible Investment/Lagged Hard Assets",
            prefix(\multicolumn{@span}{c}{) suffix(})
                span erepeat(\cmidrule(lr){@span}))
            mtitles("All" "All" "Pre-2005" "Post-2004" "All" "All")
            drop(_I* c.*)
            scalars("N Observations" "N_clust Firms" "r2 R$^2$" 
                "year Year Effects" "firm Firm Effects" "controls Controls"
                 "indy Industry Trends");        
    #delimit cr
    estimates clear

end/*%>*/

capture program drop table_bonus_robustness/*%<*/
program define table_bonus_robustness

    ** Structures.
    eststo: areg log_structures wpv_policy _Iyear*, absorb(mein) vce(cluster mein)
    estadd local indy "No": est1
    eststo: areg log_structures wpv_policy _Iyear* two_digit_naics##(c.year##c.year), absorb(mein) vce(cluster mein)
    estadd local indy "Yes": est2
    ** Hard Assets.
    eststo: areg depr_asset_growth wpv_policy _Iyear*, absorb(mein) vce(cluster mein)
    estadd local indy "No": est3
    eststo: areg depr_asset_growth wpv_policy _Iyear* two_digit_naics##(c.year##c.year), absorb(mein) vce(cluster mein)
    estadd local indy "Yes": est4
    ** Has Bonus.
    eststo: areg has_bonus wpv_policy _Iyear*, absorb(mein) vce(cluster mein)
    estadd local indy "No": est5
    eststo: areg has_bonus wpv_policy _Iyear* two_digit_naics##(c.year##c.year), absorb(mein) vce(cluster mein)
    estadd local indy "Yes": est6

    #delimit ;
    esttab using bonus-robusto-20130818.tex,
        label replace se booktabs nonumbers fragment b(a2)
            mgroups("Structures" "Net Investment" "Has Bonus",
            pattern(1 0 1 0 1 0)
            prefix(\multicolumn{@span}{c}{) suffix(})
                span erepeat(\cmidrule(lr){@span}))
            mtitles("Basic" "Trends" "Basic" "Trends" "Basic" "Trends")
            drop(_I*)
            scalars("N Observations" "N_clust Firms" "r2 R$^2$" 
                "year Year Effects" "firm Firm Effects" 
                "indy Industry Trends");
    #delimit cr
    estimates clear

end/*%>*/

capture program drop make_finsplit_markers/*%<*/
program define make_finsplit_markers
    syntax, [n(integer 10)]

    preserve
    tempfile markers
    keep if (year >= 1998 & year <= 2000) | (year >= 2005 & year <= 2007)
    bys mein: egen mean_sales = mean(grs_rcpts)
    by mein: egen mean_assets = mean(std_tot_assts)
    by mein: egen mean_pay = mean(wage_compensation)
    capture drop payer
    g payer = cond(dividends != . & corp_form != "S" & year < 2005, dividends > 0, .)
    by mein: egen ever_pay = max(payer)
    duplicates drop mein, force
    xtile sales_group = mean_sales, nq(`n')
    xtile assets_group = mean_assets, nq(10)
    xtile pay_group = mean_pay, nq(10)
    keep mein mean_sales ever_pay sales_group-pay_group
    save `markers', replace
    restore

    merge m:1 mein using `markers', keep(1 3) nogen

end/*%>*/

capture program drop make_finsplit_liq/*%<*/
program define make_finsplit_liq

    egen Icorp_form = group(corp_form)
    mkspline size_spline 10 = std_tot_assts
    qui areg liquid_assets size_spline* _Iyear* Icorp_form, absorb(four_digit)
    predict liq_r, resid
    sort mein year
    by mein: g lagged_liq = liq_r[_n-1]
    trim_tails lagged_liq, level(.01)
    xtile liq_group = lagged_liq, nq(10)
    by mein: replace liq_group = . if lagged_liq == . | std_tot_assts[_n-1] == 0

end/*%>*/

capture program drop table_bonus_finsplits/*%<*/
program define table_bonus_finsplits
    syntax, [detail]

    tempfile orig
    save `orig'
    
    make_finsplit_markers

    * Sales.
    capture drop size_group
    gen size_group = sales_group
    gen treatment = cond(size_group <= 3, 1, cond(size_group <=7 | size_group == ., ., 0))
    eststo: areg log_investment wpv_policy _Iyear* if treatment == 1, absorb(mein) vce(cluster mein)
    eststo: areg log_investment wpv_policy _Iyear* if treatment == 0, absorb(mein) vce(cluster mein)
    areg log_investment treatment##(c.wpv_policy _Iyear*), absorb(mein) vce(cluster mein)

    if "`detail'" != "" {
        
        * Only for firms with lots of data
        capture drop rowcount
        bys mein: gen rowcount = _N
        areg log_investment wpv_policy _Iyear* if treatment == 1 & rowcount >= 10, absorb(mein) vce(cluster mein)
        areg log_investment wpv_policy _Iyear* if treatment == 0 & rowcount >= 10, absorb(mein) vce(cluster mein)

        * Group by group regs
        forv i = 1/10 {
            areg log_investment wpv_policy _Iyear* if size_group == `i', absorb(mein) vce(cluster mein)
            areg investment_cst aftertax _Iyear* if size_group == `i', absorb(mein) vce(cluster mein)
        }
    }

    drop treatment size_group 

    * Dividend ever pay for C corps.
    gen treatment = ever_pay
    eststo: areg log_investment wpv_policy _Iyear* if treatment == 0, absorb(mein) vce(cluster mein)
    eststo: areg log_investment wpv_policy _Iyear* if treatment == 1, absorb(mein) vce(cluster mein)
    areg log_investment treatment##(c.wpv_policy _Iyear*), absorb(mein) vce(cluster mein)
    drop treatment

    bys mein: egen mean_logit = mean(log_investment)
    by mein: egen mean_logzt = mean(wpv_policy)
    gen logit_dm = log_investment - mean_logit
    gen logzt_dm = wpv_policy - mean_logzt
    foreach x of varlist _Iyear* {
        by mein: egen mean_`x' = mean(`x')
        gen `x'_dm = `x' - mean_`x'
        drop mean_`x'
    }

    * Liquid Assets.
    make_finsplit_liq
    gen treatment = cond(liq_group <= 3, 1, cond(liq_group <=7 | liq_group == ., ., 0))
    eststo: areg log_investment wpv_policy _Iyear* if treatment == 1, absorb(mein) vce(cluster mein)
    eststo: areg log_investment wpv_policy _Iyear* if treatment == 0, absorb(mein) vce(cluster mein)
    reg logit_dm treatment##(c.logzt_dm c._Iyear*_dm), vce(cluster mein)
    drop treatment lagged_liq* size_spline* liq_r

    #delimit ;
    esttab using bonus-hetero-20130818.tex,
        label replace se booktabs nonumbers fragment b(a2)
            mgroups("Sales" "Div Payer?" "Lagged Cash",
                pattern(1 0 1 0 1 0)
                prefix(\multicolumn{@span}{c}{) suffix(})
                    span erepeat(\cmidrule(lr){@span}))
            mtitles("Small" "Big" "No" "Yes" "Low" "High")
            scalars("N Observations" "N_clust Firms" "r2 R$^2$");
    #delimit cr
    est clear
    
    use `orig', clear

end/*%>*/

capture program drop table_bonus_taxsplits/*%<*/
program define table_bonus_taxsplits

    preserve
    ** Main regressions.
    fvset base 2000 year
    eststo: areg log_investment taxable##(c.wpv_policy year), absorb(mein) vce(cluster mein)
    eststo: areg log_investment taxable##(c.wpv_policy c.cash_flow_cst year), absorb(mein) vce(cluster mein)
    eststo: areg log_investment taxable##(c.wpv_policy year) if year < 2005, absorb(mein) vce(cluster mein)
    eststo: areg log_investment taxable##(c.wpv_policy year) if year >= 2005, absorb(mein) vce(cluster mein)
    estadd local controls "No": est*
    #delimit ;
    eststo: areg log_investment taxable##(c.wpv_policy year c.industry_q
        c.std_tot_assts##c.std_tot_assts##c.std_tot_assts##c.std_tot_assts
        c.grs_rcpts##c.grs_rcpts##c.grs_rcpts##c.grs_rcpts
        c.profit_margin##c.profit_margin##c.profit_margin##c.profit_margin
        c.age##c.age##c.age##c.age),
        absorb(mein) vce(cluster mein);
    #delimit cr
    estadd local controls "Yes": est5
    estadd local indy "No": est*
    tempfile tx
    save `tx', replace
    keep log_investment taxable wpv_policy year two_digit_naics mein
    xi i.year i.two_digit
    foreach x of varlist _Iyear* {
        gen tax_`x' = taxable*`x'
    }
    foreach x of varlist _Itwo* {
        gen `x'_year = `x'*year
        gen `x'_year2 = `x'*year*year
    }
    gen year2 = year*year
    gen taxable_z = taxable*wpv_policy
    eststo: areg log_investment taxable wpv_policy taxable_z year year2 _Iyear* tax_* _Itwo*_year*, absorb(mein) vce(cluster mein)
    estadd local indy "Yes": est6

    use `tx', clear
    * Loss carryforward guys.
    keep if has_nol & net_incm_plus > 0 & net_incm_plus != .
    g nol_income = avlbl_crryovr_nol/net_incm_plus
    trim_tails nol_income, level(.01)
    xtile nol_group = nol_income, nq(10)
    gen treatment = cond(nol_group <= 3, 1, cond(nol_group <=7 , 2, 3))
    eststo: areg log_investment treatment##(c.wpv_policy _Iyear*), absorb(mein) vce(cluster mein)
    estadd local indy "No": est7
    estadd local firm "Yes": est*
    estadd local year "Yes": est*
    #delimit ;
    esttab using bonus-indy-taxable-20130818.tex,
        label 
        replace se booktabs nonumbers fragment b(a2)
        mgroups("LHS Variable is Log(Investment)", prefix(\multicolumn{@span}{c}{) 
        suffix(})span erepeat(\cmidrule(lr){@span})) 
        mtitles("All" "All" "Pre-2005" "Post-2004" "All" "All" "LCF")
        drop(c.* std_tot_assts grs_rcpts profit_margin *199* *20* 0b.* 1o.* 
            1.taxable#c.s* 1.taxable#c.p* 1.taxable#c.g*)
        scalars("N Observations" "N_clust Firms" "r2 R$^2$" "year Year Effects" 
            "firm Firm Effects" "controls Controls" "indy Industry Trends");
    #delimit cr
    estimates clear
    restore

end/*%>*/

capture program drop table_bonus_outcomes/*%<*/
program define table_bonus_outcomes

    preserve
    local reglist rent_growth-debt_issues log_dividends payer
    ** Main regressions.
    foreach x of varlist `reglist' {
        eststo: xi: areg `x' wpv_policy i.year if big == 1, absorb(mein) vce(cluster mein)
    }
    sort mein year
    by mein: gen lag_theta2 = cond(year[_n-2] == year[_n] - 2, wpv_policy[_n-2], .)
    eststo: areg log_investment wpv_policy lag_theta2 _Iyear_*, absorb(mein) vce(cluster mein)

    estadd local firm "Yes": est*
    estadd local year "Yes": est*

    #delimit ;
    esttab using bonus-indy-outcomes-20130818.tex,
        label replace se booktabs nonumbers fragment b(a2)
            mgroups("Dependent Variable",
                prefix(\multicolumn{@span}{c}{) suffix(})
                    span erepeat(\cmidrule(lr){@span}))
            mtitles("Rents Growth" "Payroll Growth" "Debt Issues" "Dividends" "Payer?" "Lagged z")
            drop(_I*)
            scalars("N Observations" "N_clust Firms" "r2 R$^2$" 
                "year Year Effects" "firm Firm Effects");
    #delimit cr
    estimates clear
    restore
    
end/*%>*/

*******************************************************************************
* From the revision.
*******************************************************************************
capture program drop make_salesgrowth/*%<*/
program define make_salesgrowth

    capture drop delta_sales
    
    tempfile orig spine
    save `orig'
    
    * Explore missing or zero sales.
    keep mein year grs_rcpts
    gen missing_sales = grs_rcpts == .
    gen zero_sales = grs_rcpts == 0
    tabstat missing_sales zero_sales, by(year) s(mean)
    
    * Generate sales_growth and lagged sales_growth variables, 
    * accounting for potential holes in the time series.
    sort mein year
    by mein: gen sales_growth = cond(year - year[_n-1] == 1, ///
                                     log(grs_rcpts) - log(grs_rcpts[_n-1]), .)
    save `spine'
    keep mein year sales_growth
    replace year = year + 1
    rename sales_growth sales_growth_L1
    merge 1:1 mein year using `spine', keep(2 3) nogen
    save `spine', replace
    keep mein year sales_growth
    replace year = year + 2
    rename sales_growth sales_growth_L2
    merge 1:1 mein year using `spine', keep(2 3) nogen
    
    * Gen sales growth over different windows.
    gen mean_sales_growth2 = (sales_growth + sales_growth_L1)/2
    gen mean_sales_growth3 = (sales_growth + sales_growth_L1 + sales_growth_L2)/3
    
    * Explore missings
    gen msg1 = sales_growth == .
    gen msg2 = mean_sales_growth2 == .
    gen msg3 = mean_sales_growth3 == .
    tabstat msg1-msg3, by(year) s(mean)
    drop msg1-msg3
    
    * Basic stats for sales_growth by year
    tabstat sales_growth mean_sales_growth2 mean_sales_growth3, ///
        by(year) s(min p1 p10 p50 p90 p99 max mean)
    
    keep mein year sales_growth mean_sales_growth2 mean_sales_growth3
    sum sales_growth
    merge 1:1 mein year using `orig', keep(2 3) nogen
    sum sales_growth
    
    preserve
    tempfile markers
    keep if (year >= 1998 & year <= 2000) | (year >= 2005 & year <= 2007)
    bys mein: egen mean_sales_growth = mean(sales_growth)
    by mein: egen mean_msales_growth2 = mean(mean_sales_growth2)
    by mein: egen mean_msales_growth3 = mean(mean_sales_growth3)
    by mein: egen mean_age = mean(age)
    duplicates drop mein, force
    xtile sales_growth_group = mean_sales_growth, nq(10)
    xtile sales_growth2_group = mean_msales_growth2, nq(10)
    xtile sales_growth3_group = mean_msales_growth3, nq(10)
    xtile age_group = mean_age, nq(10)
    keep mein sales_growth_group-sales_growth3_group age_group
    save `markers'
    restore

    merge m:1 mein using `markers', keep(1 3) nogen

end/*%>*/

capture program drop make_spikes/*%<*/
program define make_spikes

    gen inactive = cond(investment_cst != ., investment_cst <= 0.01, .)
    gen spike = cond(investment_cst != ., investment_cst >= .2, .)
    
end/*%>*/

capture program drop make_ihat/*%<*/
program define make_ihat

    display "Prediction model"
    
    local spec1 "L.payer L.sales_growth i.sales_group i.liq_group"
    
    xtset mein year
    foreach x of varlist inactive spike {
        regress `x' `spec1' if year < 2001
        predict `x'_hat1, xb
        areg `x' `spec1' if year < 2001, absorb(four_digit)
        predict `x'_hat2, xb
    }
    
    tabstat spike spike_hat* inactive inactive_hat*, ///
        s(mean p1 p5 p10 p25 p50 p75 p90 p95 p99) by(year)
    
    forv i = 1/2 {
        xtile inactive_group`i' = inactive_hat`i', nq(10)
        tabstat inactive, by(inactive_group`i') s(mean)
        xtile spike_group`i' = spike_hat`i', nq(10)
        tabstat spike, by(spike_group`i') s(mean)
    }
    drop spike_hat* inactive_hat*
    
end/*%>*/

capture program drop table_bonus_heterolump/*%<*/
program define table_bonus_heterolump
    * Inherits from xp-20150730:
    *   xp_salesgrowth
    *   xp_predictionmodel
    syntax, [detail]

    tempfile orig
    save `orig'
    
    make_finsplit_markers
    make_finsplit_liq
    make_salesgrowth
    make_spikes
    make_ihat

    * Sales growth => Treatment == 1 for slow growers
    foreach x of varlist sales_growth_group age_group {
        capture drop size_group
        gen size_group = `x'

        gen treatment = cond(size_group <= 3, 1, cond(size_group <=7 | size_group == ., ., 0))
        eststo: areg log_investment wpv_policy _Iyear* if treatment == 1, absorb(mein) vce(cluster mein)
        eststo: areg log_investment wpv_policy _Iyear* if treatment == 0, absorb(mein) vce(cluster mein)
        areg log_investment treatment##(c.wpv_policy _Iyear*), absorb(mein) vce(cluster mein)
        drop treatment size_group 
    }

    bys mein: egen mean_logit = mean(log_investment)
    by mein: egen mean_logzt = mean(wpv_policy)
    gen logit_dm = log_investment - mean_logit
    gen logzt_dm = wpv_policy - mean_logzt
    foreach x of varlist _Iyear* {
        by mein: egen mean_`x' = mean(`x')
        gen `x'_dm = `x' - mean_`x'
        drop mean_`x'
    }

    foreach x of varlist spike_group2 inactive_group2 {
        display "--> Testing `x'"
        capture drop size_group
        gen size_group = `x'
        gen treatment = cond(size_group <= 3, 1, cond(size_group <=7 | size_group == ., ., 0))
        eststo: areg log_investment wpv_policy _Iyear* if treatment == 1, absorb(mein) vce(cluster mein)
        eststo: areg log_investment wpv_policy _Iyear* if treatment == 0, absorb(mein) vce(cluster mein)
        reg logit_dm treatment##(c.logzt_dm c._Iyear*_dm), vce(cluster mein)
        drop treatment
    }

    #delimit ;
    esttab using bonus-heterolump-20150730.tex,
        label replace se booktabs nonumbers fragment b(a2)
            mgroups("Sales Growth" "Age" "P(Spike)" "P(Inactive)",
                pattern(1 0 1 0 1 0 1 0)
                prefix(\multicolumn{@span}{c}{) suffix(})
                    span erepeat(\cmidrule(lr){@span}))
            mtitles("Low" "High" "Young" "Old" "Low" "High" "Low" "High") 
            scalars("N Observations" "N_clust Firms" "r2 R$^2$");
    #delimit cr
    est clear
    
    use `orig', clear

end/*%>*/

capture program drop make_firmz/*%<*/
program define make_firmz

    tempfile orig firmz
    save `orig'

    drop share_3yr-share_eligible
    replace basis_listed = .25*basis_listed
    egen eligible_sum = rowtotal(basis_*yr basis_listed)
    gen share_3yr = basis_3yr/eligible_sum
    gen share_5yr = basis_5yr/eligible_sum
    gen share_7yr = basis_7yr/eligible_sum
    gen share_10yr = basis_10yr/eligible_sum
    gen share_15yr = basis_15yr/eligible_sum
    gen share_20yr = basis_20yr/eligible_sum
    gen share_listed = basis_listed/eligible_sum

    gen z_firm = .947*share_3yr + .904*share_5yr + .864*share_7yr + .811*share_10yr + .694*share_15yr + .622*share_20yr + .904*share_listed

    collapse (mean) z_firm (count) rowcount=year, by(mein)
    keep z_firm rowcount mein
    save `firmz'
    
    use `orig', clear
    merge m:1 mein using `firmz', keep(1 3) nogen
    
    gen wpv_policy_firm = z_firm
    replace wpv_policy_firm = .3 + .7*z_firm if year == 2001 | year == 2002
    replace wpv_policy_firm = .5 + .5*z_firm if year == 2003 | year == 2004 | year == 2008 | year == 2009
    replace wpv_policy_firm = 1 if year == 2010 
    
    gen aftertax_firm = (1-.35*wpv_policy_firm)/(1-.35)
    
end/*%>*/

capture program drop summarize_firmz/*%<*/
program define summarize_firmz
    
    tempfile orig
    save `orig'
    
    duplicates drop mein, force
    tab rowcount
    tabstat z_firm, s(mean min p1 p5 p10 p25 p50 p75 p90 p95 p99 max)

    * How much residual variation is there?
    xi: regress z_firm i.four_digit
    
    * Correlation betweeen firm z and the number of observations in the
    * dataset.
    replace z_firm = 100*z_firm
    binscatter z_firm rowcount, reportreg savegraph("z_firm_obs.eps") replace
    
    use `orig', clear
    
end/*%>*/

capture program drop table_firmlevelz/*%<*/
program define table_firmlevelz

    tempfile orig
    save `orig'

    make_firmz
    summarize_firmz
    
    rename wpv_policy wpv_policy_indy
    rename wpv_policy_firm wpv_policy
    rename aftertax aftertax_indy
    rename aftertax_firm aftertax
    cd "$outputdir/replication/firmz"

    table_bonus_indy
    table_bonus_robustness
    table_bonus_finsplits
    table_bonus_taxsplits
    table_bonus_outcomes
    table_bonus_heterolump

    cd ..
    use `orig', clear
    
end/*%>*/

capture program drop table_hs/*%<*/
program define table_hs

    tempfile orig
    save `orig'

    merge m:1 four_digit using $analysis/industry_shares, keep(1 3) keepusing(weighted_present_value_hs)

    * Confirm baseline results
    areg log_investment wpv_policy _Iyear*, absorb(mein) vce(cluster mein)

    * Make HS covariates (see page 751).
    gen lambda = 0
    replace lambda = .3 if year == 2001 | year == 2002
    replace lambda = .5 if year == 2003 | year == 2004 | year == 2008 | year == 2009
    replace lambda = 1 if year == 2010 | year == 2011

    * HP Consumption from their dataset (see xp_replicate_20150930.do in external repo).
    gen hp_consump = .
    replace hp_consump = -.0086666 if year == 1993
    replace hp_consump = -.0079005 if year == 1994
    replace hp_consump = -.0134834 if year == 1995
    replace hp_consump = -.0157114 if year == 1996
    replace hp_consump = -.0151 if year == 1997
    replace hp_consump = -.0044292 if year == 1998
    replace hp_consump = .0061916 if year == 1999
    replace hp_consump = .0180331 if year == 2000
    replace hp_consump = .0104743 if year == 2001
    replace hp_consump = .0013373 if year == 2002
    replace hp_consump = -.0055504 if year == 2003
    replace hp_consump = -.0010965 if year == 2004
    replace hp_consump = .0006004 if year == 2005
    replace hp_consump = -.0003051 if year == 2006

    gen psi1 = (0.3425*(1-.2975)*(1-weighted_present_value)) / (1-.3425*(1-.2975)*weighted_present_value)
    gen psi1_hs = (0.3425*(1-.2975)*(1-weighted_present_value_hs)) / (1-.3425*(1-.2975)*weighted_present_value_hs)

    gen psi2 = 1 / (1 - .3425*(1-.2975)*weighted_present_value)
    gen psi2_hs = 1 / (1 - .3425*(1-.2975)*weighted_present_value_hs)

    gen lambda_psi = lambda*psi1    
    gen lambda_psi_hs = lambda*psi1_hs
    gen consump_psi = hp_consump*psi2
    gen consump_psi_hs = hp_consump*psi2_hs
    sum lambda_psi lambda_psi_hs, d

    bys year: sum lambda_psi lambda_psi_hs

    keep if year < 2007
    * Baseline
    eststo: areg log_investment lambda_psi consump_psi _Iyear*, absorb(mein) vce(cluster mein)

    * Use their Zs.
    eststo: areg log_investment lambda_psi_hs consump_psi_hs _Iyear*, absorb(mein) vce(cluster mein)

    * Big versus small
    make_finsplit_markers

    capture drop size_group
    gen size_group = sales_group
    eststo: areg log_investment lambda_psi consump_psi _Iyear* if size_group == 10, absorb(mein) vce(cluster mein)

    #delimit ;
    esttab using bonus-hs-20150730.tex,
        label replace se booktabs nonumbers fragment b(a2)
            mtitles("Baseline" "HS (2008) Zs" "Top Decile Firms") 
            scalars("N Observations" "N_clust Firms" "r2 R$^2$");
    #delimit cr
    est clear

    use `orig', clear

end/*%>*/

capture program drop make_investment_weight/*%<*/
program define make_investment_weight
    
    tempfile orig weights
    save `orig'

    * Generate investment weights.
    keep if (year >= 1998 & year <= 2000) | (year >= 2005 & year <= 2007)
    bys mein: egen mean_s179eb = mean(s179eb)
    duplicates drop mein, force
    xtile s179eb_group = mean_s179eb, nq(10)
    keep mein mean_s179eb s179eb_group
    save `weights'
    
    use `orig', clear
    merge m:1 mein using `weights', keep(1 3) nogen
    
end/*%>*/

capture program drop aggregate_prediction_coeffmath/*%<*/
program define aggregate_prediction_coeffmath

    * Prep data.
    tempfile wpv inv orig orig2
    save `orig'
    save `orig2'
    * a. Compute change in WPV and aftertax for all industries in B1 and B2.
    sum wpv_policy if year < 2001
    gen pre_b1 = `r(mean)'
    sum wpv_policy if year >= 2001 & year <= 2004
    gen b1 = `r(mean)'
    sum wpv_policy if year >= 2005 & year <= 2007
    gen pre_b2 = `r(mean)'
    sum wpv_policy if year >= 2008
    gen b2 = `r(mean)'
    sum wpv_policy if bonus == 0
    gen pre_b = `r(mean)'
    keep pre_b1-pre_b
    keep if _n == 1
    gen key = 1
    list
    save `wpv'
    
    * b. Compute share of investment in each group aggregating over all years
    *    with good weights (probably 1999-2010).
    use `orig', clear
    keep if year >= 1999
    collapse (sum) agg_s179eb = s179eb [pweight=weight_f], by(year)
    format %20.0f agg_s179eb
    list

    use `orig', clear
    gen here = 1
    keep if year >= 1999
    drop if sales_group == .
    #delimit ;
    collapse (sum) agg_s179eb = s179eb 
                   count = here [pweight=weight_f], by(sales_group);
    #delimit cr    
    egen total_s179eb = sum(agg_s179eb)
    format %20.0f agg_s179eb total_s179eb
    gen inv_share = agg_s179eb/total_s179eb
    list
    save `inv'
    
    * 1. Regression coeff times average change in wpv
    use `orig', clear
    gen key = 1
    merge m:1 key using `wpv', keep(3) nogen
    drop key
    save `orig2', replace
    
    areg log_investment wpv_policy _Iyear*, absorb(mein) vce(cluster mein)
    gen change_b1 = (b1-pre_b1)*_b[wpv_policy]
    gen change_b2 = (b2-pre_b2)*_b[wpv_policy]
    keep if _n == 1
    list b1 pre_b1 b2 pre_b2 change_b1 change_b2
    
    * 2. Math with sales group coeffs and weights based on 
    *    share of aggregate investment in each group.
    use `orig2', clear
    merge m:1 sales_group using `inv', keep(1 3) nogen
    save `orig2', replace
    
    set matsize 1000
    areg log_investment sales_group##(c.wpv_policy year), absorb(mein) vce(cluster mein)
    
    drop if sales_group == .
    gen change_hetero = inv_share*_b[wpv_policy] if sales_group == 1
    gen coeff_hetero = _b[wpv_policy]
    gen change_group_b1 = (b1-pre_b1)*coeff_hetero
    gen change_group_b2 = (b2-pre_b2)*coeff_hetero
    forv i = 2/20 {
        replace coeff_hetero = _b[wpv_policy] + _b[`i'.sales_group#c.wpv_policy] if sales_group == `i'
        replace change_hetero = inv_share*coeff_hetero if sales_group == `i'
        replace change_group_b1 = (b1-pre_b1)*coeff_hetero if sales_group == `i'
        replace change_group_b2 = (b2-pre_b2)*coeff_hetero if sales_group == `i'
    }
    duplicates drop sales_group, force
    list sales_group inv_share coeff_hetero change_hetero change_group_b1 change_group_b2
    collapse (sum) change_hetero, by(b1 pre_b1 b2 pre_b2)
    
    gen change_b1 = (b1-pre_b1)*change_hetero
    gen change_b2 = (b2-pre_b2)*change_hetero
    list

    use `orig', clear
    
end/*%>*/

capture program drop collapse_wpv/*%<*/
program define collapse_wpv

    collapse (mean) wpv_policy aftertax (p50) weighted_present_value, by(four_digit bonus)
    drop if four_digit == .
    sort four_digit bonus
    
    by four_digit: gen num_recs = _N
    display "Offending NAICS"
    list four_digit num_recs if num_recs != 2
    keep if num_recs == 2
    
    by four_digit: gen wpv_diff = wpv_policy[2] - wpv_policy[1]
    by four_digit: gen rec_num = _n
    
    keep if rec_num == 1
    sort weighted_present_value
    list four_digit weighted_present_value wpv_policy wpv_diff 
    
    xtile wpv_group = wpv_diff, nq(20)
    list four_digit wpv_diff if wpv_group == 1
    bys wpv_group: egen mean_wpv_diff = mean(wpv_diff)
    keep four_digit mean_wpv_diff wpv_group wpv_diff 

end/*%>*/

capture program drop make_wpv_groups/*%<*/
program define make_wpv_groups

    tempfile indy_b1 indy_b2 orig
    save `orig'
    
    keep if year <= 2004
    collapse_wpv
    rename mean_wpv_diff mean_wpv_diff_b1
    rename wpv_group wpv_group_b1
    rename wpv_diff wpv_diff_b1
    save `indy_b1'
    
    use `orig', clear
    merge m:1 four_digit using `indy_b1', keep(1 3) nogen
    save `orig', replace
    
    keep if year >= 2005
    collapse_wpv
    rename mean_wpv_diff mean_wpv_diff_b2
    rename wpv_group wpv_group_b2
    rename wpv_diff wpv_diff_b2
    save `indy_b2'

    use `orig', clear
    merge m:1 four_digit using `indy_b2', keep(1 3) nogen

end/*%>*/

capture program drop compute_delta_y/*%<*/
program define compute_delta_y

    tempfile orig dy_log dy_cst
    save `orig'
    
    * Log investment version.
    * Compute group Dx diffs.
    duplicates drop wpv_group, force
    drop if wpv_group == .
    sort wpv_group
    list wpv_group mean_wpv_diff
    assert _N == 20 & wpv_group[1] == 1
    
    gen delta_x = mean_wpv_diff - mean_wpv_diff[1]
    list wpv_group mean_wpv_diff delta_x
    gen delta_y = beta_log * delta_x
    list wpv_group mean_wpv_diff delta_x delta_y
    save `dy_log'
        
    * Compute aggregate level diffs.
    use `orig', clear
    drop if wpv_group == .
    collapse (sum) agg_s179eb=s179eb [pweight=weight_f], by(wpv_group)
    format %20.0f agg_s179eb
    merge 1:1 wpv_group using `dy_log', keep(3) nogen
    gen agg_s179cf = exp(-1 * delta_y) * agg_s179eb
    gen agg_idelta = agg_s179eb-agg_s179cf
    gen delta_pct = agg_idelta/agg_s179cf
    keep wpv_group agg_s179cf agg_s179eb agg_idelta delta_pct
    format %20.4f *
    sort wpv_group
    list
    collapse (sum) agg_s179cf agg_s179eb agg_idelta
    gen delta_pct = agg_idelta/agg_s179cf
    gen annual_delta = agg_idelta/4
    format %20.4f *
    list

    
end/*%>*/

capture program drop aggregate_prediction_miansufi/*%<*/
program define aggregate_prediction_miansufi

    tempfile orig
    save `orig'

    * 1. Compute deciles and average wpv_policy diffs.
    make_wpv_groups
    
    * 2. Use coeff math coefficient.
    gen beta_log = 2.8907
    save `orig', replace
    
    * 3. Compute Dy as Dx from 1 times beta from 2.
    * 4. Turn Dy into level of y using formulas.
    
    display "COUNTERFACTUAL FOR BONUS 1"
    keep if year >= 2001 & year <= 2004
    rename wpv_group_b1 wpv_group
    rename mean_wpv_diff_b1 mean_wpv_diff
    compute_delta_y

    display "COUNTERFACTUAL FOR BONUS 2"
    use `orig', clear
    keep if year >= 2005
    rename wpv_group_b2 wpv_group
    rename mean_wpv_diff_b2 mean_wpv_diff
    compute_delta_y
    
end/*%>*/

capture program drop numbers_aggregate/*%<*/
program define numbers_aggregate

    tempfile orig
    save `orig'

    set trace on
    set tracedepth 1
    make_investment_weight
    make_finsplit_markers, n(20)

    aggregate_prediction_coeffmath
    aggregate_prediction_miansufi

    use `orig', clear

end/*%>*/
*******************************************************************************
* Main
*******************************************************************************
capture program drop main
program define main
    log using "$outputdir/replication/draft-stimulus.log", replace
    cd "$outputdir/replication"

    load_bunch_data
    graph_bunch_calendar
    graph_bunch_loss
    graph_bunch_nol
    graph_bunch_bandwidth

    load_bonus_data, dset("stimulus-20130823.dta")
    graph_bonus_calendar, dset("stimulus-20130823.dta")
    graph_bonus_coeffsplit
    graph_bonus_bandwidth
    numbers_bonus
    table_bonus_sumstats
    table_bonus_indy
    table_bonus_robustness
    table_bonus_finsplits
    table_bonus_taxsplits
    table_bonus_outcomes

    table_bonus_heterolump
    table_firmlevelz
    table_hs
    numbers_aggregate

    pstopdf .

    cd $localcodedir
    log close
end

