*******************************************************************************
*     Title: external-replication.do
*   Purpose: Replicate House and Shapiro and run analysis in Compustat, based
*            on Edgerton paper.
*     Owner: Eric Zwick 
*       Eds: Tom Cui
*      Date: 2016-08-01
*******************************************************************************

global datadir "/path/to/data"
global outdir "/path/to/output"

* Library
do "/path/to/library/programs.do"

********************************************************************************
* Data builds
********************************************************************************

capture program drop load_hs_orig/*%<*/
program define load_hs_orig

    import delimited "$datadir/HouseShapiro2008", clear

    **let Stata know this is a factor variable
    encode industry_type, gen(Indsty_type)


    **setup time
    gen tim = date(time, "YMD")

    gen Time = qofd(tim)
    gen year = year(tim)

    format Time %tqCYq

    drop time tim industry_type

    order Indsty_type Time

    save "$datadir/HouseShapiro2008", replace

end/*%>*/

capture program drop load_hs_annual/*%<*/
program define load_hs_annual
    
    load_hs_orig

    keep if year >= 1970 & year != 2007

    * All data are annualized. 
    * Investment data are in Millions, everything else in Billions
    * See BEA tables 5.4.5.AU etc to compare.
    #delimit ;
    collapse 
        (mean) real_investment
        (mean) real_consump
        (mean) real_prices
        (mean) realgdp
        (mean) realcorporateprofits
        (mean) itc
        (mean) psi1 psi2 bonus_tax_subsidy
        , by(year Indsty_type);
    #delimit cr

    save "$datadir/HouseShapiro2008_annual", replace

end/*%>*/

capture program drop load_edge/*%<*/
program define load_edge

    import delimited "$datadir/compustat_edgerton_20151015.csv", clear
    #delimit;
    rename (v1-v21) 
        (gvkey year fyr eins naics
         capx ppent prcc csho at ceq txdb ib dp 
         pi xido itci tlcf txfed sale ppegt);
    #delimit cr

    gen log_investment = log(capx)

    destring naics, replace force
    gen four_digit_naics = round(naics/100)

	preserve
	tempfile markers
	keep if (year >= 1998 & year <= 2000) | (year >= 2005 & year <= 2007)
    keep if sale > 0 & at > 0
    gen ebitda = ib+dp
	bys gvkey: egen mean_ebitda = mean(ebitda)
    bys gvkey: egen mean_assets = mean(at)
    bys gvkey: egen mean_sales = mean(sale)
	duplicates drop gvkey, force
    sum sale, d
	xtile ebitda_group = mean_ebitda, nq(10)
    xtile at_group = mean_assets, nq(10)
    xtile sales_group = mean_sales, nq(10)
	keep gvkey mean_ebitda ebitda_group mean_assets at_group mean_sales sales_group
	save `markers'
	restore

	merge m:1 gvkey using `markers', keep(1 3) nogen
end/*%>*/

********************************************************************************
* HS computations
********************************************************************************

capture program drop make_hs_orig/*%<*/
program define make_hs_orig

    load_hs_orig

    **set the panel accordingly
    tsset Indsty_type Time


    **truncating to years and industries used in the paper
    keep if Time >= tq(1965,1) & Time <= tq(2006,4)

    drop if Indsty_type==34 | Indsty_type==37

    *the early series for computers was "too volatile"
    drop if Indsty_type==6 & Time < tq(1970,1)


    **switch to logs
    gen log_invest = log(real_investment)

    gen log_prc = log(real_prices)

    gen logGDP = log(realgdp)
     
    gen logconsump = log(real_consump)

    gen logprofits = log(realcorporateprofits)


    **HP Filter real consumption
    tsfilter hp hp_consump = logconsump



    **Loops and creates 24*2 forecast variables
    *truncates the series so we don't regress past 2000 values

    gen fore_inv = log_invest if Time <= tq(2000,4)

    gen fore_prc = log_prc if Time <= tq(2000,4)

    *quietly{
    foreach H of numlist 1/24 {
        by Indsty_type: gen fore_inv`H' = fore_inv[_n+`H']
        by Indsty_type: gen fore_prc`H' = fore_prc[_n+`H']
    }
    *}

    *wiping data so we have two columns for just the forecasts
    replace fore_inv = .

    replace fore_prc = .

    *Create the lags used in paper
    by Indsty_type: gen inv_lag = log_invest[_n-1]

    by Indsty_type: gen prc_lag = log_prc[_n-1]

    by Indsty_type: gen gdp_lag = logGDP[_n-1]

    by Indsty_type: gen itc_lag = itc[_n-1]

    by Indsty_type: gen pi_lag = logprofits[_n-1]


    **Running the forecasts on real investment
    **Idea: we're running 24*36 regressions. Each regression is run on
    **the dataset from 60s to 2000, dependent variable is a H-period lead,
    **H= 1...24. So we're simulating type-specific trends for 2001 to
    **2006, when people start accounting for the tax cut policy.
    **See Sec III.B of the paper.

    *Macro allows iteration over types (have to store values so no by:() ) 
    levelsof Indsty_type, local(type)

    *loop over types, then forecasting period
    *quietly{
    foreach H of numlist 1/24{ 
        gen resid`H' = .
        foreach i of local type {
            *forecasting equation, two lags
            reg fore_inv`H' log_invest log_prc logGDP logprofits itc ///
                inv_lag prc_lag gdp_lag itc_lag pi_lag Time ///
                if Indsty_type == `i'
            *predicting a single datapoint from 2000:IV stats
            _predict Forecast if Indsty_type == `i' & Time == 163, xb
            *that was saved in 2000:IV row so push it forward
            gen Forecast2 = Forecast[_n-`H']
            *pumping out the errors
            _predict resid if Indsty_type == `i'
            *24*36 error vectors in all, but arrange in panel format
            replace resid`H' = resid if Indsty_type == `i' & !missing(fore_inv`H')
            *store into fore_inv, up to now all missing values
            replace fore_inv = Forecast2 if Indsty_type == `i' & Time == 163+`H'
            drop Forecast Forecast2 resid

        }
    }


    *calculate deviations from actual values
    replace fore_inv = log_invest - fore_inv



    ***very messy code for calculating the covariance matrix below

    foreach i of local type {
        correlate resid1-resid24 if Indsty_type==`i', cov
        matrix residmtx`i' = r(C)
        matrix residmtx`i' = residmtx`i' / el(residmtx`i', 1, 1)
    }

    *Taking the mean

    matrix residR = residmtx1

    foreach i of numlist 2/33 35 36 38{
        matrix residR = residmtx`i' + residR
    }

    matrix residR = residR/36

    drop resid2-resid24

    separate resid1, by(Indsty_type)

    foreach i of numlist 1/6 {
        replace resid1`i' = resid1`i'[_n+168*(`i'-1)]
    }

    replace resid17 = resid17[_n+168*5+148]

    foreach i of numlist 8/33 {
        replace resid1`i' = resid1`i'[_n+148+168*(`i'-2)]
    }

    replace resid135 = resid135[_n+148+168*32]
    replace resid136 = resid136[_n+148+168*33]
    replace resid138 = resid138[_n+148+168*34]

    corr resid11-resid138, cov

    matrix residS = r(C)

    set matsize 900

    matrix resid_Inv = residS # residR

    drop resid1 resid11-resid138

    **And again for prices
    foreach H of numlist 1/24{ 

        gen resid`H' = .
        foreach i of local type {
            reg fore_prc`H' log_invest log_prc logGDP logprofits itc ///
                inv_lag prc_lag gdp_lag itc_lag pi_lag Time ///
                if Indsty_type == `i'
            _predict Forecast if Indsty_type == `i' & Time == 163, xb
            gen Forecast2 = Forecast[_n-`H']
            _predict resid if Indsty_type == `i'
            replace resid`H' = resid if Indsty_type == `i' & !missing(fore_prc`H')
            replace fore_prc = Forecast2 if Indsty_type == `i' & Time == 163+`H'
            drop Forecast Forecast2 resid
        }

    }

    replace fore_prc = log_prc - fore_prc


    foreach i of local type {
        correlate resid1-resid24 if Indsty_type==`i', cov
        matrix residmtx`i' = r(C)
        matrix residmtx`i' = residmtx`i' / el(residmtx`i', 1, 1)
    }

    matrix residR = residmtx1

    foreach i of numlist 2/33 35 36 38{
        matrix residR = residmtx`i' + residR
    }

    matrix residR = residR/36

    drop resid2-resid24

    separate resid1, by(Indsty_type)

    foreach i of numlist 1/6 {
        replace resid1`i' = resid1`i'[_n+168*(`i'-1)]
    }

    replace resid17 = resid17[_n+168*5+148]

    foreach i of numlist 8/33 {
        replace resid1`i' = resid1`i'[_n+148+168*(`i'-2)]
    }

    replace resid135 = resid135[_n+148+168*32]
    replace resid136 = resid136[_n+148+168*33]
    replace resid138 = resid138[_n+148+168*34]

    corr resid11-resid138, cov

    matrix residS = r(C)

    set matsize 900

    matrix resid_prc = residS # residR




    drop if missing(fore_inv)

    gen var_Inv = .

    gen var_prc = .


    foreach i of numlist 1/864{
        replace var_Inv in `i' = sqrt(el(resid_Inv, `i', `i'))
        replace var_prc in `i' = sqrt(el(resid_prc, `i', `i'))

    }



    *}

    **Now we have 864 values in fore_inv and fore_prc.
    **generate the covariates of second-stage regression

    gen x1 = psi1*bonus_tax_subsidy

    gen x2 = psi2*hp_consump

    *vwls fore_inv x1 x2, sd(var_Inv)


    **Export to TeX file
    *build the esttab table
    eststo reg_inv: reg fore_inv x1 x2

    binscatter fore_inv x1, control(x2) reportreg ///
        savegraph("$outdir/foreinv_binscatter_qtr.pdf") replace

end/*%>*/

capture program drop make_hs_annualized/*%<*/
program define make_hs_annualized

    load_hs_annual

    **set the panel accordingly
    tsset Indsty_type year

    keep if year >= 1965 & year <= 2006

    drop if Indsty_type==34 | Indsty_type==37

    *the early series for computers was "too volatile"
    drop if Indsty_type==6 & year < 1970

    **switch to logs
    gen log_invest = log(real_investment)

    gen log_prc = log(real_prices)

    gen logGDP = log(realgdp)
     
    gen logconsump = log(real_consump)

    gen logprofits = log(realcorporateprofits)

    **HP Filter real consumption
    tsfilter hp hp_consump = logconsump

    **Loops and creates 24*2 forecast variables
    *truncates the series so we don't regress past 2000 values

    gen fore_inv = log_invest if year <= 2000

    gen fore_prc = log_prc if year <= 2000

    foreach H of numlist 1/6 {
        by Indsty_type: gen fore_inv`H' = fore_inv[_n+`H']
        by Indsty_type: gen fore_prc`H' = fore_prc[_n+`H']
    }

    *wiping data so we have two columns for just the forecasts
    replace fore_inv = .

    replace fore_prc = .

    *Create the lags used in paper
    by Indsty_type: gen inv_lag = log_invest[_n-1]

    by Indsty_type: gen prc_lag = log_prc[_n-1]

    by Indsty_type: gen gdp_lag = logGDP[_n-1]

    by Indsty_type: gen itc_lag = itc[_n-1]

    by Indsty_type: gen pi_lag = logprofits[_n-1]

    **Running the forecasts on real investment
    **Idea: we're running 6*36 regressions. Each regression is run on
    **the dataset from 60s to 2000, dependent variable is a H-period lead,
    **H= 1...6. So we're simulating type-specific trends for 2001 to
    **2006, when people start accounting for the tax cut policy.
    **See Sec III.B of the paper.

    *Macro allows iteration over types (have to store values so no by:() ) 
    levelsof Indsty_type, local(type)

    *loop over types, then forecasting period
    foreach H of numlist 1/6{ 
        gen resid`H' = .
        foreach i of local type {
            *forecasting equation, two lags

            reg fore_inv`H' log_invest log_prc logGDP logprofits itc ///
                inv_lag prc_lag gdp_lag itc_lag pi_lag year ///
                if Indsty_type == `i'

            *predicting a single datapoint from 2000:IV stats
            _predict Forecast if Indsty_type == `i' & year == 2000, xb //Time == 163, xb
            *that was saved in 2000:IV row so push it forward
            gen Forecast2 = Forecast[_n-`H']
            *pumping out the errors
            _predict resid if Indsty_type == `i'
            *24*36 error vectors in all, but arrange in panel format
            replace resid`H' = resid if Indsty_type == `i' & !missing(fore_inv`H')
            *store into fore_inv, up to now all missing values
            replace fore_inv = Forecast2 if Indsty_type == `i' & year == 2000+`H' // Time == 163+`H'
            drop Forecast Forecast2 resid
        }
    }

    *calculate deviations from actual values
    replace fore_inv = log_invest - fore_inv

    ***very messy code for calculating the covariance matrix below
    * Omitted

    **generate the covariates of second-stage regression

    gen x1 = psi1*bonus_tax_subsidy

    gen x2 = psi2*hp_consump

    *************************************************************
    * Compare to quarterly regs
    * reg_inv                                            
    * b                7.034284     10.03937    -.2134748
    *************************************************************
    eststo reg_inv_annual: reg fore_inv x1 x2

    * Exclude some industries with special Zs
    * 
    * Single-family structures (34), Farm tractors (13), Petroleum and natural
    * gas (27), Mining (20), Other agricultural machinery (24)
    drop if Indsty_type == 34 | Indsty_type == 13 | Indsty_type == 27 | ///
        Indsty_type == 20 | Indsty_type == 24

    reg fore_inv x1 x2
    binscatter fore_inv x1, control(x2) reportreg ///
        savegraph("$outdir/foreinv_binscatter_annual.pdf") replace

    * Excluding the consumption term
    reg fore_inv x1

    * Exclude ineligibles
    reg fore_inv x1 x2 if psi1 < .1

    * Exclude 20yr structures
    binscatter fore_inv x1 if psi1 < .09, control(x2) reportreg ///
        savegraph("$outdir/foreinv_binscatter_annual_15orless.pdf") replace

end/*%>*/

capture program drop xp_hs_depreciation/*%<*/
program define xp_hs_depreciation

    load_hs_orig

    tempfile orig
    save `orig'

    * Keep a bonus year and check out the parameters
    keep if year == 2003
    duplicates drop Indsty_type, force

    gsort - zvalue
    list Indsty_type bonus_subsidy zvalue jorgenson_z psi1 psi2

    * They have psi1s for each type (see Table A2 in the paper)
    * 
    * Method       psi1        z_value      Our z
    * 5yr 200DB    0.0318407   0.8962004    0.904
    * 7yr 200DB    0.041587    0.8630491    0.864
    * 10yr 200DB   0.0547588   0.8171598    0.811
    * 15yr 150DB   0.0826337   0.7157013    0.694
    * 20yr 150DB   0.0996784   0.6505672    0.622
    * 39yr SL      0.1512963   0.4373577

    * These are the industries with the really high x1
    /*
        Communication structures 
        Electric structures 
        Farm structures 
        Internal combustion engines 
        Other power structures  
        Railroads structures   
    */

    use `orig', clear

end/*%>*/

********************************************************************************
* Appendix tables with external data
********************************************************************************
capture program drop appendix_ptrends/*%<*/
program define appendix_ptrends

    tempfile wpv edge

    * Load WPV data
    insheet using $datadir/naics_wpv_20130901.csv, clear tab
    save `wpv'

    load_edge
    merge m:1 four_digit_naics using `wpv', keep(3) nogen

    * Generate policy variable
    gen wpv_policy = weighted_present_value
    replace wpv_policy = .3 + .7*weighted_present_value if year == 2001 | year == 2002
    replace wpv_policy = .5 + .5*weighted_present_value if year == 2003 | year == 2004 | year == 2008 | year == 2009
    replace wpv_policy = 1 if year == 2010 | year == 2011
    replace wpv_policy = .5 + .5*weighted_present_value if year == 2012 | year == 2013 | year == 2014

    gen aftertax = (1-.35*wpv_policy)/(1-.35)
    save `edge'

    * Summarize data.
    keep if year >= 1993 & year <= 2010
    xi i.year

    * Main regression
    eststo: areg log_investment wpv_policy _Iyear*, absorb(gvkey) cluster(gvkey)

    * Size splits
	capture drop size_group
	gen size_group = sales_group 
    gen treatment = cond(mean_sales <= 164, 1, cond(mean_sales == ., ., 0))
	eststo: areg log_investment wpv_policy _Iyear* if treatment == 1, absorb(gvkey) vce(cluster gvkey)
	eststo: areg log_investment wpv_policy _Iyear* if treatment == 0, absorb(gvkey) vce(cluster gvkey)
	areg log_investment treatment##(c.wpv_policy _Iyear*), absorb(gvkey) vce(cluster gvkey)
    areg log_investment size_group##(c.wpv_policy _Iyear*), absorb(gvkey) vce(cluster gvkey)
    tabstat mean_sales, by(size_group) s(mean)
    tabstat mean_sales, by(treatment) s(mean)
	drop treatment size_group 

    * Previous recessions.
    use `edge', clear

    * 1. Start with log_inv and wpv_policy set up using 1997-2004 and 2005-2010
    keep if year >= 2005 & year <= 2014
    xi i.year
    eststo: areg log_investment wpv_policy _Iyear*, absorb(gvkey) vce(cluster gvkey)

    use `edge', clear
    keep if year >= 1996 & year <= 2005
    xi i.year
    eststo: areg log_investment wpv_policy _Iyear*, absorb(gvkey) vce(cluster gvkey)

	estadd local firm "Yes": est*
	estadd local year "Yes": est*
	#delimit ;
	esttab using $outdir/bonus-compustat-20150930.tex,
		label replace se booktabs nonumbers fragment b(a2)
			mgroups("LHS Variable is Log(Capital Expenditures)",
			prefix(\multicolumn{@span}{c}{) suffix(})
				span erepeat(\cmidrule(lr){@span}))
			mtitles("1993-2010" "Small" "Big" "R2008" "R2001")
			drop(_I*)
			scalars("N Observations" "N_clust Firms" "r2 R$^2$" 
				"year Year Effects" "firm Firm Effects");
	#delimit cr
	estimates clear

    * 2. Set up 1990 recession using an 8-year window. Regen wpv_policy as
    *    above.
    * 1990 recession: July 1990 March 1991
    use `edge', clear
    keep if year >= 1985 & year <= 1994
    xi i.year
    replace wpv_policy = .3 + .7*weighted_present_value if year == 1990 | year == 1991
    replace wpv_policy = .5 + .5*weighted_present_value if year == 1992 | year == 1993 
    eststo: areg log_investment wpv_policy _Iyear*, absorb(gvkey) vce(cluster gvkey)

    * 1980 recession: January 1980  July 1980 July 1981   November 1982
    use `edge', clear
    keep if year >= 1975 & year <= 1984
    xi i.year
    replace wpv_policy = .3 + .7*weighted_present_value if year == 1980 | year == 1981
    replace wpv_policy = .5 + .5*weighted_present_value if year == 1982 | year == 1983 
    eststo: areg log_investment wpv_policy _Iyear*, absorb(gvkey) vce(cluster gvkey)

    * November 1973 March 1975
    use `edge', clear
    keep if year >= 1968 & year <= 1977
    xi i.year
    replace wpv_policy = .3 + .7*weighted_present_value if year == 1973 | year == 1974
    replace wpv_policy = .5 + .5*weighted_present_value if year == 1975 | year == 1976
    eststo: areg log_investment wpv_policy _Iyear*, absorb(gvkey) vce(cluster gvkey)

    * December 1969 November 1970
    use `edge', clear
    keep if year >= 1964 & year <= 1973
    xi i.year
    replace wpv_policy = .3 + .7*weighted_present_value if year == 1969 | year == 1970
    replace wpv_policy = .5 + .5*weighted_present_value if year == 1971 | year == 1972
    eststo: areg log_investment wpv_policy _Iyear*, absorb(gvkey) vce(cluster gvkey)

    * December 1960 February 1961
    use `edge', clear
    keep if year >= 1955 & year <= 1964
    xi i.year
    replace wpv_policy = .3 + .7*weighted_present_value if year == 1960 | year == 1961
    replace wpv_policy = .5 + .5*weighted_present_value if year == 1962 | year == 1963
    eststo: areg log_investment wpv_policy _Iyear*, absorb(gvkey) vce(cluster gvkey)

	estadd local firm "Yes": est*
	estadd local year "Yes": est*
	#delimit ;
	esttab using $outdir/bonus-ptrends-20150930.tex,
		label replace se booktabs nonumbers fragment b(a2)
			mgroups("LHS Variable is Log(Capital Expenditures)",
			prefix(\multicolumn{@span}{c}{) suffix(})
				span erepeat(\cmidrule(lr){@span}))
			mtitles("R1990" "R1980" "R1973" "R1969" "R1960")
			drop(_I*)
			scalars("N Observations" "N_clust Firms" "r2 R$^2$" 
				"year Year Effects" "firm Firm Effects");
	#delimit cr
	estimates clear

end/*%>*/

capture program drop appendix_hs/*%<*/
program define appendix_hs

    make_hs_orig
    make_hs_annualized
    #delimit ;
    esttab using "$outdir/hs_orig_20150930.tex",
        label replace se booktabs nonumbers fragment b(a2)
        scalars("N Observations" "r2 R$^2$");
    #delimit cr
    estimates clear

end/*%>*/

********************************************************************************
* MAIN
********************************************************************************

capture program drop main
program define main

    capture log c
    log using $outdir/external-replication.log, replace
    set trace on
    set tracedepth 1

    * Appendix tables
    appendix_ptrends
    appendix_hs

end
