*******************************************************************************
*   Title: make-stimulus.do
* Purpose: Produces data sets for final published version
*   Owner: Eric Zwick
*    Date: 2016-08-01
*******************************************************************************

*******************************************************************************
* External datasets and loads
*******************************************************************************
capture program drop build_soi_indy_naics
program define build_soi_indy_naics

	display "BUILD soi_indy_naics"
	insheet using $raw/soi_indy_naics.csv, clear delim(",")	
	save $analysis/soi_indy_naics.dta, replace
	
end

capture program drop build_section179
program define build_section179
	
	display "BUILD section179"
	insheet using $raw/section_179_new.csv, clear delim(",")
	rename bonus bonus_deduction
	save $analysis/section_179.dta, replace

end

capture program drop build_industryQ
program define build_industryQ
	
	display "BUILD industryQ"
	insheet using $raw/industryQ_compustat.csv, clear tab	
	rename four_digit four_digit_naics
	rename fyear year
	save $analysis/industryQ.dta, replace

end

capture program drop build_weights
program define build_weights
	
	display "BUILD weights"
	insheet using $raw/soi_weights.csv, clear comma
	rename weight_f weight_f_soi
	save $analysis/soi_weights.dta, replace
	
end

capture program drop load_investment
program define load_investment

	insheet using $raw/investment.csv, clear delim(";")
	
	* Recoding some tax years.
	* 1994 and 1998 are funky
	replace end_acct_yr = end_acct_yr + 2 if end_acct_yr > 90 & end_acct_yr < 100 & (year == 1994 | year == 1998)
	* 90s are coded with two-digit years. Weak.
	replace end_acct_yr = 1900 + end_acct_yr if end_acct_yr > 0 & end_acct_yr < 100
	sort mein year
	by mein: replace end_acct_yr = 1994 if year == 1994 & end_acct_mon == 0 & end_acct_yr[_n-1] == 1993
	by mein: replace end_acct_yr = 1995 if year == 1994 & end_acct_mon == 0 & end_acct_yr[_n-1] == 1994
	by mein: replace end_acct_yr = 1998 if year == 1998 & end_acct_mon == 0 & end_acct_yr[_n-1] == 1997
	by mein: replace end_acct_yr = 1999 if year == 1998 & end_acct_mon == 0 & end_acct_yr[_n-1] == 1998
	
	* Grab the new year when it's filled
	replace year = end_acct_yr if end_acct_yr > 1985 & end_acct_yr < 2020
	
end

capture program drop load_bunch
program define load_bunch

	load_investment
	
end
*******************************************************************************
* Selecting rows
*******************************************************************************
capture program drop filter_returns
program define filter_returns
		
	keep if active_firm & full_year_return
	
	* Delete rows.	
	* Drop partial year returns.
	drop if end_acct_mon > 12 & end_acct_mon < .
	
	duplicates drop year mein, force
	
	keep if corp_form == "C1" | corp_form == "C2" | corp_form == "S"

	merge 1:1 ein year end_acct_mon using "$analysis/soi_weights.dta", keep(1 3) nogen
	rename weight_f weight_f_ota	
	gen weight_f = cond(weight_f_soi != ., weight_f_soi, weight_f_ota)
	
	gen bad_weight = weight_f < 1
	tabstat bad_weight, by(year) s(mean)	
	drop if weight_f < 0
	drop bad_weight
	
end

capture program drop filter_investment
program define filter_investment
	
	drop if year > 2010
	
end

*******************************************************************************
* Adding columns and building main analysis datasets
*******************************************************************************
capture program drop gen_returns
program define gen_returns
	
	merge m:1 year using $analysis/section_179.dta, keep(3)
	drop _merge

	* Rather than total investment, let's just use section 179 eligible stuff.
	* The adjustment is to correct for the distinction between used and used/new.
	g s179eb = bonus_eligible_basis + .06*sec179_totalcost
	
	gen rep = sec179_propinservice > 0 & sec179_propinservice != . & basis_listed > cutoff
	replace adjusted_basis = adjusted_basis - s179eb + sec179_propinservice if rep
	replace s179eb = sec179_propinservice if rep
	
	* Industry codes.
	merge m:1 soi_indy_cd using $analysis/soi_indy_naics
	keep if _merge == 1 | _merge == 3
	drop _merge
	gen four_digit_naics = .
	replace four_digit_naics = soi_indy_cd if soi_indy_cd/10000 < 1
	replace four_digit_naics = int(naics/100) if four_digit_naics != . & naics != . & year < 1999
	replace four_digit_naics = int(soi_indy_cd/100) if soi_indy_cd/100000 > 1 & soi_indy_cd != . & year >= 1999
	drop naics
	
	* Industry recoding.
	gen four_digit_sub = .
	replace four_digit_sub = four_digit_naics if year > 2001
	bys ein: egen fds_mode = mode(four_digit_sub)
	replace four_digit_sub = fds_mode if year <= 2001
	replace four_digit_naics = four_digit_sub if year <= 2001 & four_digit_sub != .
	drop four_digit_sub
	
	* General columns.
	g has_nol = avlbl_crryovr_nol > 0 & avlbl_crryovr_nol != .
	g taxable = net_incm_plus > 0

end

capture program drop gen_dfl
program define gen_dfl
	* syntax dfl_group_var bin_varlist base_group count_var
	
	tempfile orig work wpv baseGroup
	save `orig'
	
	* Generate industry types.
	use $analysis/industry_shares_earlyonly, clear
	xtile wpv_treatment2 = weighted_present_value, nq(10)
	keep four_digit_naics wpv_treatment2
	save `wpv'

	use `orig', clear

	merge m:1 four_digit_naics using `wpv', keep(3) nogen
	gen treatment = cond(wpv_treatment2 <= 3, 1, cond(wpv_treatment2 >= 8, 0, .))
	replace treatment = . if big != 1
	
	* Make size bin.
	tab wpv_treatment2, m
	xtile sales_bin = grs_rcpts if treatment != ., nq(10)
	assert sales_bin == . if treatment == .
	xtile assets_bin = std_tot_assts if treatment != ., nq(10)
	egen dfl_bin = group(sales_bin assets_bin)
	drop sales_bin assets_bin
	save `work'
	
	* Generate base group counts.
	keep if year == 2000 & treatment == 1
	collapse (count) base_bin_count=ein if dfl_bin != ., by(dfl_bin)
	egen base_group_count = sum(base_bin_count)
	save `baseGroup'

	use `work', clear
	merge m:1 dfl_bin using `baseGroup', keep(1 3) nogen

	* Generate own counts.
	egen own_bin_count = count(ein), by(dfl_bin year treatment)
	egen own_group_count = count(ein) if dfl_bin != ., by(year treatment)
	gen dfl_weight = (base_bin_count/own_bin_count) * (own_group_count/base_group_count)
	assert dfl_weight == . if treatment == .
	drop own_bin_count own_group_count base_bin_count base_group_count treatment dfl_bin

end

capture program drop gen_investment
program define gen_investment

	* Merge in industry instrument and industry Q
	merge m:1 four_digit using $analysis/industry_shares_earlyonly.dta, keep(1 3) nogen
	merge m:1 four_digit year using $analysis/industryQ.dta, keep(1 3) nogen
	gen two_digit_naics = int(four_digit_naics/100)

	capture drop age
	gen age = .
	replace age = year - date_inc
	replace age = . if date_inc == 0 | date_inc == 9999

	xtile size_group = std_tot_assts, nq(20)
	
	gen profit_margin = net_incm_plus/grs_rcpts
	gen roa = net_incm_plus/std_tot_assts
	
	g log_investment = log(s179eb)
	g has_eligible = s179eb > 0
	replace has_eligible = . if s179eb == .
	bys four_digit year: egen count_on = sum(has_eligible)
	bys four_digit year: egen count_off = sum(abs(has_eligible-1))
	gen log_odds = log(count_on/count_off)

	g log_payroll = log(wage_compensation)
	g log_dividends = log(dividend)
	g payer = dividend > 0
	replace payer = . if dividend == .
	gen log_structures = log(structures)
	sort mein year
	by mein: gen depr_asset_growth = cond(year == year[_n-1] + 1, log(dprcbl_assts) - log(dprcbl_assts[_n-1]), .)
	gen has_bonus = bonus_depreciation > 0
	assert bonus_depreciation != .
	
	sort mein year
	by mein: gen rent_growth = cond(year == year[_n-1] + 1, log(rnts_pd) - log(rnts_pd[_n-1]), .)
	by mein: gen payroll_growth = cond(year == year[_n-1] + 1, log_payroll - log_payroll[_n-1], .)
	by mein: gen debt_issues = cond(year == year[_n-1] + 1, log(total_debt) - log(total_debt[_n-1]), .)	
	
	by mein: gen investment_cst = cond(year - year[_n-1] == 1, s179eb/dprcbl_assts[_n-1], .)
	by mein: gen cash_flow_cst = cond(year - year[_n-1] == 1, (net_incm_plus-tx_pd)/std_tot_assts[_n-1], .)
	
	by mein: egen mean_investment_h = mean(s179eb) if s179eb > 0
	by mein: egen mean_investment = max(mean_investment_h)
	replace mean_investment = 0 if mean_investment == .
	gen big = mean_investment > 100
	
	xi i.year
	
	list in 1
	gen bonus = (year == 2001 | year == 2002 | year == 2003 | year == 2004 | year == 2008 | year == 2009 | year == 2010 | year == 2011)
	gen wpv_policy = weighted_present_value
	replace wpv_policy = .3 + .7*weighted_present_value if year == 2001 | year == 2002
	replace wpv_policy = .5 + .5*weighted_present_value if year == 2003 | year == 2004 | year == 2008 | year == 2009
	replace wpv_policy = 1 if year == 2010 | year == 2011
	gen wfd_policy = weighted_firstded
	replace wfd_policy = .3 + .7*weighted_firstded if year == 2001 | year == 2002
	replace wfd_policy = .5 + .5*weighted_firstded if year == 2003 | year == 2004 | year == 2008 | year == 2009
	replace wfd_policy = 1 if year == 2010 | year == 2011
	gen aftertax = (1-.35*wpv_policy)/(1-.35)
	
	* Winsor and trim
	impose_bounds profit_margin, lower(-10) upper(10)
	impose_bounds roa, lower(-10) upper(10)
	impose_bounds book_leverage, lower(0) upper(5)
	trim_tails investment_cst, level(.01)
	trim_tails cash_flow_cst, level(.01)
	
	gen_dfl
	
end

capture program drop build_investment
program define build_investment
	
	display "BUILD investment"
	load_investment
	filter_returns
	gen_returns
	gen_investment
	filter_investment
	
	#delimit ;
	keep year std_tot_assts corp_form
		net_dpr size_group mein ein end_acct_mon
		bonus_depreciation sec179_depreciation sec179_totalcost 
		sec179_propinservice sec179_listedprop listedprop_depr
		weight_f
		adjusted_basis
		v_state_cd zip
		structures hard_assets bonus_eligible_basis
		wage_compensation intrst_pd liquid_assets dividends lns_frm_stkhdr grs_rcpts
		total_debt broad_equity rnts_pd book_leverage
		net_incm_plus tot_ded cmpnstn_offcr retained_earnings free_cash_flow tx_pd
		dprcbl_assts
		avlbl_crryovr_nol frgo_crrybck_nol nold
		roa profit_margin net_working_capital trd_notes_accts_rcv accts_pybl
		s179eb cutoff phaseout
		basis_3yr-basis_listed age		
		log_investment log_odds investment_cst cash_flow_cst big
		wpv_policy wfd_policy aftertax 
		wpv_treatment2 dfl_weight 
		_Iyear* two_digit_naics four_digit industry_q
		log_payroll log_structures depr_asset_growth rent_growth payroll_growth debt_issues
		taxable has_bonus has_nol log_dividends payer has_eligible
	;
	#delimit cr
	save $analysis/ibonus.dta, replace
	
end

capture program drop gen_bunch
program define gen_bunch

	gen basis_part = s179eb - int(s179eb)
	* Super-fine bins.
	#delimit ;
	gen basis_sec = cond(basis_part < .125, 0, 
						cond(basis_part < .375, .25,
						cond(basis_part < .625, .5,
						cond(basis_part < .875, .75, 1))));
	#delimit cr	
	gen basis_bin = int(s179eb) + basis_sec
	gen basis_bin_diff = basis_bin - cutoff				

end

capture program drop build_bunch
program define build_bunch
	
	display "BUILD bunch"
	load_bunch
	filter_returns
	gen_returns
	gen_bunch
	
	#delimit ;
	keep year std_tot_assts corp_form
		net_dpr mein ein
		bonus_depreciation sec179_depreciation sec179_totalcost 
		sec179_propinservice sec179_listedprop listedprop_depr
		weight_f
		adjusted_basis
		v_state_cd zip
		structures hard_assets bonus_eligible_basis
		wage_compensation intrst_pd liquid_assets dividends lns_frm_stkhdr grs_rcpts
		total_debt broad_equity rnts_pd book_leverage
		net_incm_plus tot_ded cmpnstn_offcr retained_earnings free_cash_flow tx_pd
		dprcbl_assts
		avlbl_crryovr_nol frgo_crrybck_nol nold
		net_working_capital trd_notes_accts_rcv accts_pybl four_digit
		s179eb basis_bin basis_bin_diff cutoff phaseout
		basis_3yr-basis_listed
	;
	#delimit cr
	save $analysis/ibunch.dta, replace
	
end

********************************************************************************
* Derived analysis datasets
********************************************************************************
capture program drop collapse_indy
program define collapse_indy

	collapse share_*, by(four_digit)
	********************************************************************************
	*	Source is Treasury report to Congress on depreciation (2000), table 8. 
	*	Alternatively, we could use IRS Pub 946 or House and Shapiro (2008), 
	*	table A.1.
	*******************************************************************************
	gen weighted_present_value = .947*share_3yr + .904*share_5yr + .864*share_7yr + ///
		.811*share_10yr + .694*share_15yr + .622*share_20yr + .904*share_listed
	gen weighted_firstded = .3333*share_3yr + .2*share_5yr + .1429*share_7yr + ///
		.1*share_10yr + .05*share_15yr + .0375*share_20yr + .2*share_listed
end

capture program drop build_industry_aggs
program define build_industry_aggs

	display "BUILD industry aggs"
	
	use $analysis/ibunch.dta, clear
	
	replace basis_listed = .25*basis_listed
	egen eligible_sum = rowtotal(basis_*yr basis_listed)
	gen share_3yr = basis_3yr/eligible_sum
	gen share_5yr = basis_5yr/eligible_sum
	gen share_7yr = basis_7yr/eligible_sum
	gen share_10yr = basis_10yr/eligible_sum
	gen share_15yr = basis_15yr/eligible_sum
	gen share_20yr = basis_20yr/eligible_sum
	gen share_listed = basis_listed/eligible_sum
	gen share_eligible = s179eb/adjusted_basis
	
	* Exclude bonus years.
	tempfile orig
	save `orig'
	
	drop if (year >= 2001 & year < 2005) | (year >= 2008)
	collapse_indy
	save $analysis/industry_shares.dta, replace
	
	use `orig', clear
	drop if year >= 2001
	collapse_indy
	save $analysis/industry_shares_earlyonly.dta, replace
	
end

capture program drop outsheet_failure
program define outsheet_failure

	load_investment
	tempfile orig
	save `orig'
	
	duplicates drop mein, force
	rename mein tin
	keep tin
	outsheet using $raw/tin_failure.csv, comma replace
	
	use `orig', clear
	keep if year > 1998
	duplicates drop mein end_acct_yr, force
	rename mein tin
	rename end_acct_yr tax_yr
	keep tin tax_yr
	outsheet using $raw/tin_failure_acctyr.csv, comma replace
	
	cd $raw
	!zip tin_failure tin_failure.csv tin_failure_acctyr.csv
	cd $localcodedir
	
end

*******************************************************************************
* Main
*******************************************************************************
capture program drop main
program define main
	log using "$outputdir/replication/make-stimulus.log", replace

	**********************
	* External datasets
	**********************
	build_soi_indy_naics
	build_section179
	build_industryQ
	build_weights
	
	**********************
	* Main analysis dataset
	**********************
	build_bunch

	**********************
	* Derived analysis dataset
	* Uses build_bunch output.
	**********************
	build_industry_aggs

	**********************
	* Main analysis dataset
	* Uses build_industry_aggs output.
	**********************
	build_investment

	**********************
	* Derived dataset with failure rates.
	**********************
	outsheet_failure
	
	log close
end
