*******************************************************************************
*   Title: programs.do
* Purpose: Project specific programs
*   Owner: Eric Zwick and James Mahon
*    Date: 2016-08-01
*******************************************************************************

capture program drop bunch_pic
program define bunch_pic
	syntax varname [if/], lower(real) upper(real) cutoff(real) year(string) [type(string)]
	preserve 
	if "`if'" != "" {
		keep if `if'
	}
	if `cutoff' < 75 {
		drop basis_bin
		gen basis_part = `varlist' - int(`varlist')
		gen basis_sec = cond(basis_part < .25, 0, cond(basis_part < .75, .5, 1))
		gen basis_bin = int(`varlist') + basis_sec
		drop basis_part basis_sec
	}
	else {
		drop basis_bin
		gen basis_bin = round(`varlist')
	}
	if "`type'" == "mean" {
		collapse (mean) `varlist', by(basis_bin)
	}
	else {
		collapse (count) `varlist', by(basis_bin)
	}
	#delimit;
	twoway scatter `varlist' basis_bin if basis_bin >= `lower' & basis_bin <= `upper', 
		connect(l) msymbol(O) xline(`cutoff')
		ytitle("Number of Firms") xtitle("Section 179 Eligible Investment (000s)")
		mfcolor(green) lcolor(black*.25);
	#delimit cr
	graph export "`varlist'-bunch-`year'.eps", replace
	restore
end

capture program drop bunch_split
program define bunch_split
    syntax varname
    collapse (count) freq=basis_bin, by(basis_bin_diff `varlist')
    #delimit ;
    twoway scatter freq basis_bin_diff 
        if basis_bin_diff > -10 & basis_bin_diff < 10, 
        by(`varlist')
        connect(l) msymbol(O)
        ytitle("Number of Firms") xtitle("Section 179 Eligible Investment Around Cutoff (000s)")
		mfcolor(green) lcolor(black*.25);
    #delimit cr
    graph export "s179eb-bunch-`varlist'.eps", replace

    /* For NBER digest
    keep if basis_bin_diff > -10 & basis_bin_diff < 10
    keep freq basis_bin_diff `varlist'
    outsheet using "bunch_counts.csv", replace
    */
end

capture program drop bunch_est
program define bunch_est
	syntax [if/], mark(string) [by(name) binwidth(real 250) delta(real 3) plot(integer 0) xlabel(string)]
	display "BUNCH START `mark'"
	capture gen bbd_dollars = basis_bin_diff*1000
	if "`if'" != "" {
		#delimit ;
		bunch_count bbd_dollars freq if `if', bpoint(0) binwidth(`binwidth')
			low_bunch(-`delta') high_bunch(`delta') nboot(1000);
		#delimit cr
	}
	else if "`by'" != "" {
		egen bc_group = group(`by')
		qui sum bc_group
		gen bc_bunch = .
		gen bc_se = .
		forvalues i=1/`r(max)' {
			tab `by' if bc_group == `i'
			#delimit ;
			bunch_count bbd_dollars freq if bc_group == `i', bpoint(0) binwidth(`binwidth')
				low_bunch(-`delta') high_bunch(`delta') nboot(1000);
			#delimit cr
			replace bc_bunch = `r(b2)' if bc_group == `i'
			replace bc_se = `r(b_se2)' if bc_group == `i'
		}
		if `plot' != 0 {
			collapse bc_bunch (sum) count=freq, by(`by')
			#delimit;
			twoway (scatter bc_bunch `by', mfcolor(green) msymbol(O))
				(lfit bc_bunch `by' [weight=count]),
				legend(off)
				xtitle("Quantile of `xlabel'") ytitle("Excess Mass");				
			#delimit cr
			graph export "bc_bunch-`mark'-`by'.eps", replace
			sort `by'
			list
		}
	}
	else {
		#delimit ;
		bunch_count bbd_dollars freq, bpoint(0) binwidth(`binwidth')
			low_bunch(-`delta') high_bunch(`delta') nboot(1000);
		#delimit cr
	}
	display "BUNCH END"
end

capture program drop ddplot_yagan
program define ddplot_yagan
	syntax varlist(fv) [if/] [aweight], mark(string) ///
		[by(name) absorb(name) xlabel(string) ylabel(string) raj(integer 0) ///
			diag(integer 0) start(integer 2000) end(integer 2010) trim(real 0)]
	preserve
	if "`if'" != "" {
		keep if `if'
	}
	if `trim' != 0 {
		trim_tails `1', level(`trim')
	}
	gen treatment_coeff = .
	if "`absorb'" != "" {
		forvalues i = `start'/`end' {
			display `i'
			areg `varlist' if year == `i' [`weight' `exp'], absorb(`absorb')
			matrix B = e(b)
			replace treatment_coeff = B[1,1] if year == `i'
		}
	}
	else {
		forvalues i = `start'/`end' {
			display `i'
			reg `varlist' if year == `i' [`weight' `exp']
			matrix B = e(b)
			replace treatment_coeff = B[1,1] if year == `i'
		}
	}
	bys year: egen mean_icst = mean(`1')
	bys year: egen count_total = count(mein)
	gen is_s = corp_form == "S"
	gen count = 1
	trim_tails std_tot_assts, level(.005)
	trim_tails grs_rcpts, level(.005)
	collapse (sum) std_tot_assts grs_rcpts is_s count [`weight' `exp'], by(year `2' mean_icst treatment_coeff count_total)
	gen investment_group = (count_total*mean_icst - count*treatment_coeff)/count_total + treatment_coeff if `2' == 1
	replace investment_group = (count_total*mean_icst - (count_total - count)*treatment_coeff)/count_total if `2' == 0
	
	if `raj' != 0 {
		gen mean_holder = cond(year < `start' + `raj', investment_group, .)
		bys `2': egen mean_ig = mean(mean_holder)
		egen mean_overall = mean(mean_ig)
		replace investment_group = investment_group - mean_ig + mean_overall
		sort year
	}
	
	if `diag' != 0 {
		foreach y of varlist std_tot_assts grs_rcpts is_s {
			gen `y'_wt = `y'/count
			#delimit ;
			twoway
				(connect `y'_wt year if `2' == 1,
					mcolor(red*1.5) lpattern(dash) lcolor(red*.75) msymbol(o))
				(connect `y'_wt year if `2' == 0,
					msymbol(x) lpattern(dash) mcolor(blue) lcolor(blue*.5)) if year >= `start',
				ytitle("`y'_wt")
				legend(label(1 "Treatment Group") label(2 "Control Group")) xsize(6);
			graph export "`1'-ddyagan-diag-`y'-`mark'.eps", replace;
			#delimit cr
		}
		#delimit ;
			twoway
				(connect count year if `2' == 1,
					mcolor(red*1.5) lpattern(dash) lcolor(red*.75) msymbol(o))
				(connect count year if `2' == 0,
					msymbol(x) lpattern(dash) mcolor(blue) lcolor(blue*.5)) if year >= `start',
				ytitle("count")
				legend(label(1 "Treatment Group") label(2 "Control Group")) xsize(6);
			graph export "`1'-ddyagan-diag-count-`mark'.eps", replace;
			#delimit cr
	}
	#delimit ;
	twoway 
		(connect investment_group year if `2' == 1, 
			mcolor(red*1.6) lpattern(dash) lcolor(red*.8) msymbol(O)) 
		(connect investment_group year if `2' == 0, 
			msymbol(s) lpattern(dash) mcolor(blue*1.6) lcolor(blue*.8)) 
		if year >= `start' & year <= `end', 
		xscale(r(`start' `end')) 
		xtitle("Year") ytitle("`ylabel'") 
		xline(2000.5 2007.5, lcolor(gs9))
		legend(label(1 "Treatment Group (Long Duration Industries)") 
			label(2 "Control Group (Short Duration Industries)")) xsize(6);
	graph export "`1'-ddyagan-`mark'.eps", replace;
	#delimit cr
	restore
end

